import shutil
from collections import namedtuple
from pathlib import Path

import pytest
from click.testing import CliRunner

TEST_DIR = Path(__file__).parent.resolve()
TEST_DATA_DIR = TEST_DIR / "__test_data"

TEST_FILENAMES = [
    "blah.csv",
    "blah2.csv",
    "0.csv",
    "1.csv",
    "2.csv",
    "3.csv",
    "4.csv",
    "5.csv",
]


TEST_DICT = {
    "key": "my hovercraft is full of eels",
    "key_hashed": "cfb8533d92515b5f85814ecf2f4df1e2f1068d275e807f69b1c1792af7aefc06",
    "csv_name": "test.csv",
    "csv_path": Path("test.csv"),
    "csv_content": "\n".join(("spam,eggs", "0,3", "3,4", "")),
    "hashed_name": "hashed/test.csv",
    "hashed_path": Path("hashed/test.csv"),
    "hashed_content": "\n".join(
        (
            "spam,eggs",
            "897e11d4ae3c0c9f5411c4cf66447e0da5f997618737442495e94932480b77cf,00b467a3785761d11781f10ae5ac7d474a9090f1da4287ae641dabfba7b409f8",
            "00b467a3785761d11781f10ae5ac7d474a9090f1da4287ae641dabfba7b409f8,e329703e9d16f2f4a0931d475157850f957495f95cde9d8ae9961ae284fd2d49",
        )
    ),
}

StringSack = namedtuple("StringSack", TEST_DICT.keys())  # type: ignore


@pytest.fixture
def TEST():
    return StringSack(**TEST_DICT)


def clean_local_testing():
    if TEST_DATA_DIR.exists():
        print(f"Deleting {TEST_DATA_DIR}")
        shutil.rmtree(TEST_DATA_DIR)
    print(f"Creating {TEST_DATA_DIR}")
    TEST_DATA_DIR.mkdir()
    for fn in TEST_FILENAMES:
        path = TEST_DATA_DIR / fn
        path.write_text(ARBITRARY_CSV_TEXT)
        print(f"         {path}")


@pytest.fixture
def run_runner(TEST):
    def true_runner(
        func,
        args=None,
        key=TEST.key,
        csv_name="test.csv",
        csv_content=TEST.csv_content,
        ini_name=None,
        ini_content="key={}",
    ):
        args = args or []

        if not key:
            ini_name = "secret.ini"
        if key and not ini_name:
            args.append(f"--key={key}")
        runner = CliRunner()
        with runner.isolated_filesystem():
            if ini_name:
                Path(ini_name).write_text(ini_content.format(key))
            Path(csv_name).write_text(csv_content)
            print(args)
            result = runner.invoke(func, [csv_name, *args])
            try:
                hashed = Path("./hashed", csv_name).read_text()
            except FileNotFoundError:
                hashed = None
        return result, hashed

    return true_runner


if __name__ == "__main__":
    clean_local_testing()
