from pathlib import Path

import pytest
from click.testing import CliRunner

from hmac4csv.cli import run, KEY_ENV_VAR


@pytest.mark.parametrize(
    "csv_content, shared_key",
    [
        ["spam,eggs\n1,2\n3,4\n", "spameggspamspambaconspam"],
        [
            "spam,eggs\n1,2\n3,4\n",
            "ce16aa894a567de44d158296c6030d40924d8a7f88ba04cef4fe43bc12a74916",
        ],
    ],
)
def test_cli_end_to_end_including_prehashed(csv_content, shared_key):
    runner = CliRunner()
    with runner.isolated_filesystem():
        test_csvs = [Path(f"test{i}.csv") for i in (0, 1, 2)]
        for test_csv in test_csvs:
            test_csv.write_text(csv_content)

        # As file
        Path("secret.ini").write_text(f"key={shared_key}")
        result = runner.invoke(run, str(test_csvs[2]))
        assert result.exit_code == 0
        Path("secret.ini").unlink()

        # As CLI
        result = runner.invoke(run, [str(test_csvs[0]), f"--key={shared_key}"])
        assert result.exit_code == 0

        # As env
        result = runner.invoke(run, str(test_csvs[1]), env={KEY_ENV_VAR: shared_key})
        assert result.exit_code == 0

        assert (
            len(set(Path(f"./hashed/test{i}.csv").read_text() for i in (0, 1, 2))) == 1
        )
